#include "planning.hpp"
#include "cstdlib" // Added to be able to use atof, converting character string to float
#include "string"
#include "vector"
#include <iostream>

using namespace r7_planning;

void PlanningClass::basicInfo() {
  // Printing reference frame
  ROS_INFO("!!!! NB !!!!");
  ROS_INFO("NB! Reference frame: %s", move_group.getPlanningFrame().c_str());
  ROS_INFO("!!!! NB !!!!");
  // robot_state::RobotStatePtr roboten = move_group.getCurrentState();
}

void PlanningClass::goToPoseGoal() {
  move_group.setPoseTarget(target_pose1);
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);

  if (!success) // execute
    throw std::runtime_error("No plan found");

  sleep(2.0);

  move_group.move(); // blocking
}

void PlanningClass::goToPoseGoal(geometry_msgs::Pose &pose) {
  move_group.setPoseTarget(pose);
  ros::Duration(0.5).sleep();
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);
  // moveit::planning_interface::MoveItErrorCode::SUCCESS  Changed with the
  // other package as above
  /*while (!success) //keep trying until a plan is found
  {

      success = (move_group.plan(my_plan) ==
  moveit::planning_interface::MoveItErrorCode::SUCCESS);
  }*/
  if (!success) // execute
    throw std::runtime_error("No plan found");

  move_group.move(); // blocking
}

// //This must be fixed
// void PlanningClass::goToJointState() {
// From the PR2 tutorial:
// std::vector<double> group_variable_values;
// move_group.getCurrentState()->copyJointGroupPositions(move_group.getCurrentState()->getRobotModel()->getJointModelGroup(move_group.getName()),
// group_variable_values);

//   robot_state::RobotState current_state = *move_group.getCurrentState();
//   // moveit::core::RobotStatePtr current_state =
//   move_group.getCurrentState(); std::vector<double> joint_positions;
//   joint_model_group = current_state.getJointModelGroup(PLANNING_GROUP);
//           joint_positions[0] = -1.0;
//   joint_positions[3] = 0.7;

//   move_group.setJointValueTarget(joint_positions);
//   bool success = (move_group.plan(my_plan)
//   ==moveit::planning_interface::MoveItErrorCode::SUCCESS); if (!success)
//     throw std::runtime_error("No plan found");

//   move_group.move(); // blocking
// }

void PlanningClass::cartesianPath() {
  std::vector<geometry_msgs::Pose> waypoints;
  // waypoints.push_back(target_pose1);
  // move_group.setStartStateToCurrentState();

  // Adding the initial state, hope it might work
  // geometry_msgs::Pose initPose;
  // initPose = move_group.getCurrentPose().pose;
  // waypoints.push_back(initPose);

  // From point 1.1, added orientation and position tolerance to the move_group,
  // and changed PID controllers
  move_group.setGoalOrientationTolerance(0.5);
  move_group.setGoalPositionTolerance(0.2);
  move_group.setMaxVelocityScalingFactor(0.5);

  //   // Adding a few more points
  //   geometry_msgs::Pose target_pose2;
  //   geometry_msgs::Pose target_pose3;
  //   geometry_msgs::Pose target_pose4;

  //   target_pose2.orientation.w = 0.001;
  //   target_pose2.orientation.x = 0.0;
  //   target_pose2.orientation.y = 1.0;
  //   target_pose2.orientation.z = 0.0;
  //   target_pose2.position.x = 0.3;
  //   target_pose2.position.y = 0.3;
  //   target_pose2.position.z = 0.1;

  //   target_pose3.orientation.w = 0.001;
  //   target_pose3.orientation.x = 0.0;
  //   target_pose3.orientation.y = 1.0;
  //   target_pose3.orientation.z = 0.0;
  //   target_pose3.position.x = 0.4;
  //   target_pose3.position.y = 0.0;
  //   target_pose3.position.z = 0.1;

  //   target_pose4.orientation.w = 0.001;
  //   target_pose4.orientation.x = 0.0;
  //   target_pose4.orientation.y = 1.0;
  //   target_pose4.orientation.z = 0.0;
  //   target_pose4.position.x = 0.3;
  //   target_pose4.position.y = -0.3;
  //   target_pose4.position.z = 0.1;

  //   // waypoints.push_back(target_pose1);
  //   waypoints.push_back(target_pose2);
  //   waypoints.push_back(target_pose3);
  //   waypoints.push_back(target_pose4);

  //#####################################################################################
  // Testing all of the imported/read poses fed directly to see if the reading
  // might be wrong.
  geometry_msgs::Pose target_poseX;
  target_poseX.position.x = 0.0;
  target_poseX.position.y = 0.4;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 2.75545529808154e-17;
  target_poseX.position.y = 0.45;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.139057647468726;
  target_poseX.position.y = 0.427975432332819;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.264503363531613;
  target_poseX.position.y = 0.364057647468726;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.364057647468726;
  target_poseX.position.y = 0.264503363531613;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.427975432332819;
  target_poseX.position.y = 0.139057647468726;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.45;
  target_poseX.position.y = 0;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.427975432332819;
  target_poseX.position.y = -0.139057647468726;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.364057647468726;
  target_poseX.position.y = -0.264503363531613;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.264503363531613;
  target_poseX.position.y = -0.364057647468726;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.139057647468726;
  target_poseX.position.y = -0.427975432332819;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  // Her kan det vere noke muffins (mellom 13 og 14) iom at dei har same
  // verdiane! #### DEVIL IS IN THE DETAILS #### DEILIG!!!!!!!!
  //   target_poseX.position.x = 2.77555756156289e-17;
  //   target_poseX.position.y = -0.35;
  //   target_poseX.position.z = 0.1;
  //   target_poseX.orientation.w = 0.0;
  //   target_poseX.orientation.x = 6.12323399573676e-17;
  //   target_poseX.orientation.y = 1.0;
  //   target_poseX.orientation.z = 1.11022302462516e-16;
  //   waypoints.push_back(target_poseX);

  target_poseX.position.x = 2.14313189850787e-17;
  target_poseX.position.y = -0.35;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.108155948031232;
  target_poseX.position.y = -0.332869780703304;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.205724838302366;
  target_poseX.position.y = -0.283155948031232;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.283155948031232;
  target_poseX.position.y = -0.205724838302366;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.332869780703304;
  target_poseX.position.y = -0.108155948031232;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.35;
  target_poseX.position.y = 0.0;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.332869780703304;
  target_poseX.position.y = 0.108155948031232;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.283155948031232;
  target_poseX.position.y = 0.205724838302366;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.205724838302366;
  target_poseX.position.y = 0.283155948031232;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 0.108155948031232;
  target_poseX.position.y = 0.332869780703304;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  target_poseX.position.x = 2.77555756156289e-17;
  target_poseX.position.y = 0.35;
  target_poseX.position.z = 0.1;
  target_poseX.orientation.w = 0.0;
  target_poseX.orientation.x = 6.12323399573676e-17;
  target_poseX.orientation.y = 1.0;
  target_poseX.orientation.z = 1.11022302462516e-16;
  waypoints.push_back(target_poseX);

  //######################################################################################

  // Printing the waypoints positions one by one
  for (geometry_msgs::Pose tp : waypoints) {
    ROS_INFO("Pose: %f, %f, %f", tp.position.x, tp.position.y, tp.position.z);
    ROS_INFO("Quarterion: %f, %f, %f, %f", tp.orientation.w, tp.orientation.x,
             tp.orientation.y, tp.orientation.z);
  }

  // We want the Cartesian path to be interpolated at a resolution of 1 cm
  // which is why we will specify 0.01 as the max step in Cartesian
  // translation.  We will specify the jump threshold as 0.0, effectively
  // disabling it. Warning - disabling the jump threshold while operating real
  // hardware can cause large unpredictable motions of redundant joints and
  // could be a safety issue
  // moveit_msgs::RobotTrajectory trajectory;

  const double jump_threshold = 0.0;
  const double eef_step = 0.01;
  bool avoid_collisions = false;
  double fraction = move_group.computeCartesianPath(
      waypoints, eef_step, jump_threshold, PlanningClass::trajectory,
      avoid_collisions);
  ROS_INFO_STREAM("Percentage of path followed: " << 100 * fraction);

  // Sending trajectory data to file
  std::ofstream newFile;
  newFile.open("trajectoryPre.csv");
  const char sep = ',';

  if (!newFile) {
    ROS_WARN("Cannot open file..");
  }
  ROS_INFO("File is open, ready to write.");

  // This has information related to each joint, NOT within the cartesian space
  int count = 0;
  for (trajectory_msgs::JointTrajectoryPoint jtp :
       PlanningClass::trajectory.joint_trajectory.points) {
    std::vector<double> accs =
        jtp.accelerations; // Comes from jtp.accelerations as float64[], as
                           // array
    std::vector<double> pos =
        jtp.positions; // Comes from jtp.positions as float64[], an array
    std::vector<double> eff =
        jtp.effort; // Comes from jtp.effort as float64[], an array
    std::vector<double> vel =
        jtp.velocities; // Comes from jtp.velocities as float64[], an array

    double secs = ros::Time::now().toSec();
    ros::Duration rd = jtp.time_from_start;
    secs = rd.toSec();
    ROS_INFO("Point nr. %i, Time: %f , Accs: %f %f %f %f %f %f, Pos: %f %f %f "
             "%f %f %f, Eff: %lu, Vel: %f %f %f %f %f %f",
             count, secs, accs[0], accs[1], accs[2], accs[3], accs[4], accs[5],
             pos[0], pos[1], pos[2], pos[3], pos[4], pos[5], eff.size(), vel[0],
             vel[1], vel[2], vel[3], vel[4], vel[5]);
    count += 1;
    newFile << std::to_string(pos[0]) << sep << std::to_string(pos[1]) << sep
            << std::to_string(pos[2]) << sep << std::to_string(pos[3]) << sep
            << std::to_string(pos[4]) << sep << std::to_string(pos[5])
            << ", Timestamp:" << secs << "\n";
  }

  sleep(5);
  my_plan.trajectory_ = PlanningClass::trajectory;

  // move_group.plan(my_plan); // ##### BIG BAAAAD WOLD RIGHT HERE #### DO NOT,
  // I REPEAT, DO NOT USE THIS IN ADDITION TO THE TRAJECTORY IMPLEMENTATION
  // ABOVE.

  if (fraction == 1.0) {
    move_group.execute(my_plan);
    ROS_INFO("All good, 100 percent of path was created");
  } else {
    ROS_WARN("Could not compute the cartesian path :( ");
    ROS_WARN("Make sure that app poses are feasible and that the initial pose "
             "is well-defined!");
  }
  sleep(5);

  newFile.close();
  ros::shutdown();
  // ros::Duration(4).sleep(); //Is this the way to sleep or same same?
  // or move_group.execute(&trajectory); //or the one beneath?
  // move_group.move();
}

// #### Function for creating trajectory from Matlab points input ####
void PlanningClass::cartesianPathMatlab(std::string &fil) {
  std::vector<geometry_msgs::Pose> waypoints;

  move_group.setGoalOrientationTolerance(0.5);
  move_group.setGoalPositionTolerance(0.3);
  move_group.setMaxVelocityScalingFactor(0.8);
  move_group.setPlannerId("RRTkConfigDefault"); // Trying to change planner...

  // Reading information from csv file from Matlab
  std::ifstream myFile; // Reading input file
  std::string path = "/home/user/catkin_ws/src/motion_planning/src/" + fil;
  myFile.open(path);
  std::string px, py, pz, ow, ox, oy, oz;
  double fpx, fpy, fpz, fow, fox, foy, foz;

  if (!myFile.is_open()) {
    ROS_WARN("ERROR: File Open");
  }

  while (myFile.good()) {
    geometry_msgs::Pose wp;
    std::getline(myFile, px, ',');
    std::getline(myFile, py, ',');
    std::getline(myFile, pz, ',');
    std::getline(myFile, ow, ',');
    std::getline(myFile, ox, ',');
    std::getline(myFile, oy, ',');
    std::getline(myFile, oz, '\n');
    // ROS_INFO("####Pose: %s, %s, %s; %s, %s, %s, %s", px.c_str(), py.c_str(),
    // pz.c_str(), ow.c_str(), ox.c_str(), oy.c_str(), oz.c_str());

    wp.position.x = atof(px.c_str());
    wp.position.y = atof(py.c_str());
    wp.position.z = atof(pz.c_str());
    wp.orientation.w = atof(ow.c_str());
    wp.orientation.x = atof(ox.c_str());
    wp.orientation.y = atof(oy.c_str());
    wp.orientation.z = atof(oz.c_str());
    waypoints.push_back(wp);
  }

  // Writing the input waypoints to another file to check if something might be
  // wrong with formatting.
  std::ofstream utwp;
  utwp.open("utwp.csv");

  // Printing the waypoints positions one by one
  for (geometry_msgs::Pose tp : waypoints) {
    ROS_INFO("!!! NB !!! Pose: %f, %f, %f", tp.position.x, tp.position.y,
             tp.position.z);
    ROS_INFO("!!! NB !!! Quarterion: %f, %f, %f, %f", tp.orientation.x,
             tp.orientation.y, tp.orientation.z, tp.orientation.w);

    utwp << std::to_string(tp.position.x) << "," << tp.position.y << ","
         << tp.position.z << "," << tp.orientation.w << "," << tp.orientation.x
         << "," << tp.orientation.y << "," << tp.orientation.z << "\n";
  }

  // moveit_msgs::RobotTrajectory trajectory;
  const double jump_threshold = 0.0;
  const double eef_step = 0.01;
  bool avoid_collisions = false;
  double fraction = move_group.computeCartesianPath(
      waypoints, eef_step, jump_threshold, PlanningClass::trajectory,
      avoid_collisions);
  ROS_INFO_STREAM("Percentage of path followed: " << 100 * fraction);

  // #### Trajectory information ####

  // vector of vectors for each of joint angle corresponding to point of
  // trajectory
  std::vector<std::vector<double>> anglepoints;

  // Sending trajectory data to file
  std::ofstream newFile;
  newFile.open("trajectory.csv");
  const char sep = ',';

  if (!newFile) {
    ROS_WARN("Cannot open file..");
  }
  ROS_INFO("File is open, ready to write.");

  // This has information related to each joint, NOT within the cartesian space
  int count = 0;
  ros::Duration dt(0.0);
  ros::Duration increase(0.01);
  for (trajectory_msgs::JointTrajectoryPoint jtp :
       PlanningClass::trajectory.joint_trajectory.points) {
    std::vector<double> accs =
        jtp.accelerations; // Comes from jtp.accelerations as float64[], as
                           // array
    std::vector<double> pos =
        jtp.positions; // Comes from jtp.positions as float64[], an array
    std::vector<double> eff =
        jtp.effort; // Comes from jtp.effort as float64[], an array
    std::vector<double> vel =
        jtp.velocities; // Comes from jtp.velocities as float64[], an array

    double secs = ros::Time::now().toSec();
    ros::Duration rd = jtp.time_from_start;
    secs = rd.toSec();
    // ROS_INFO("Point nr. %i, Time: %f , Accs: %f %f %f %f %f %f, Pos: %f %f %f
    // "
    //          "%f %f %f, Eff: %lu, Vel: %f %f %f %f %f %f",
    //          count, secs, accs[0], accs[1], accs[2], accs[3], accs[4],
    //          accs[5], pos[0], pos[1], pos[2], pos[3], pos[4], pos[5],
    //          eff.size(), vel[0], vel[1], vel[2], vel[3], vel[4], vel[5]);
    count += 1;
    anglepoints.push_back(pos);
    newFile << std::to_string(pos[0]) << sep << std::to_string(pos[1]) << sep
            << std::to_string(pos[2]) << sep << std::to_string(pos[3]) << sep
            << std::to_string(pos[4]) << sep << std::to_string(pos[5])
            << ", Timestamp:" << secs << "\n";

    // ## Testing to add timestamp to each waypoint manually since they are
    // defined as not strictly increasing. Increaseing by 0.01
    jtp.time_from_start = dt;
    dt += increase;
    // ROS_INFO("Timestamp new: %f", dt.toSec());
  }

  // PlanningClass::trajectory.joint_trajectory.points[PlanningClass::trajectory.joint_trajectory.points.size_t()]

  // #### Execution ####
  sleep(5);
  my_plan.trajectory_ = PlanningClass::trajectory;
  ROS_INFO("Trajectory uploaded.");

  if (fraction == 1.0) {
    move_group.execute(my_plan);
    ROS_INFO("All good, 100 percent of path was created");
  } else {
    ROS_WARN("Could not compute the cartesian path :( ");
    ROS_WARN("Make sure that app poses are feasible and that the initialpose "
             "is well-defined!");

    // sleep(5); // Added sleep to see the message better.

    // Trying to add the movement to the first waypoint
    ROS_INFO("Moving to initial state of waypoints which is as follows:");
    ROS_INFO("Pos: %f ,%f, %f and Ori: %f, %f, %f, %f", waypoints[1].position.x,
             waypoints[1].position.y, waypoints[1].position.z,
             waypoints[1].orientation.w, waypoints[1].orientation.x,
             waypoints[1].orientation.y, waypoints[1].orientation.z);
    goToPoseGoal(waypoints[0]);
    sleep(3);
    waypoints.erase(waypoints.begin());

    // #### Replanning from the new waypoints ####
    // const double jump_threshold = 0.0;
    // const double eef_step = 0.01;
    // bool avoid_collisions = false;
    double frac = move_group.computeCartesianPath(
        waypoints, eef_step, jump_threshold, PlanningClass::trajectory,
        avoid_collisions);
    ROS_INFO_STREAM("Percentage of new path followed: " << 100 * fraction);

    sleep(3);
    my_plan.trajectory_ = PlanningClass::trajectory;

    if (frac == 1.0) {
      move_group.execute(my_plan);
      ROS_INFO("All good, 100 percent of path was created");
    } else {
      ROS_WARN("Could not compute the cartesian path :( ");
      ROS_WARN("Make sure that all poses are feasible");
    }
    newFile.close();
    myFile.close();
    sleep(5);
  }

  // // #### If nothing works, try to walk through to each angle of the joints
  // (of the trajectory generated) ####

  //   int cunt = 0;
  //   for (std::vector<double> joint_positions : anglepoints) {
  //     move_group.setJointValueTarget(joint_positions);
  //     ROS_INFO("Joint angles for traj part %i of
  //     %lu",cunt,anglepoints.size()); bool success =
  //         (move_group.plan(my_plan) ==
  //         moveit_msgs::MoveItErrorCodes::SUCCESS);
  //     if (!success)
  //       throw std::runtime_error("No plan found");
  //     sleep(2);
  //     move_group.move(); // blocking
  //   }

  // Instead of the above it is possible to iterate through each 10th or
  // something joint angles.
  //   for (std::size_t it = 0; it < anglepoints.size(); it += 10) {
  //     move_group.setJointValueTarget(anglepoints[it]);
  //     ROS_INFO("Joint angles for traj part %lu of %lu", it,
  //     anglepoints.size()); bool success =
  //         (move_group.plan(my_plan) ==
  //         moveit_msgs::MoveItErrorCodes::SUCCESS);
  //     if (!success)
  //       throw std::runtime_error("No plan found");
  //     sleep(2);
  //     move_group.move(); // blocking
  //   }

  ros::shutdown();
}

// Testing of poses from the Matlab inputs
void PlanningClass::runPointsMatlab() {

  std::vector<geometry_msgs::Pose> waypoints;
  // Reading information from csv file from Matlab
  std::ifstream myFile; // Reading input file
  myFile.open("/home/user/catkin_ws/src/motion_planning/src/feed_traj.csv");
  std::string px, py, pz, ow, ox, oy, oz;
  double fpx, fpy, fpz, fow, fox, foy, foz;
  std::vector<double> posen{fpx, fpy, fpz, fow, fox, foy, foz};

  if (!myFile.is_open()) {
    ROS_WARN("ERROR: File Open");
  }

  while (myFile.good()) {
    geometry_msgs::Pose wp;
    std::getline(myFile, px, ',');
    std::getline(myFile, py, ',');
    std::getline(myFile, pz, ',');
    std::getline(myFile, ow, ',');
    std::getline(myFile, ox, ',');
    std::getline(myFile, oy, ',');
    std::getline(myFile, oz, '\n');
    // ROS_INFO("####Pose: %s, %s, %s; %s, %s, %s, %s", px.c_str(),
    // py.c_str(), pz.c_str(), ow.c_str(), ox.c_str(), oy.c_str(),
    // oz.c_str());

    wp.position.x = atof(px.c_str());
    wp.position.y = atof(py.c_str());
    wp.position.z = atof(pz.c_str());
    wp.orientation.w = atof(ow.c_str());
    wp.orientation.x = atof(ox.c_str());
    wp.orientation.y = atof(oy.c_str());
    wp.orientation.z = atof(oz.c_str());
    waypoints.push_back(wp);
  }

  // Running the points one by one.
  int nr = 0;

  for (geometry_msgs::Pose poss : waypoints) {
    goToPoseGoal(poss);
    ROS_INFO("Sleeping before next pose. Please wait...");
    ROS_WARN("This is Pose nr: %i", nr);
    sleep(2.0);
    nr++;
  }
}

void PlanningClass::resetValues() {
  // set the start state and operational speed
  move_group.setStartStateToCurrentState();
  move_group.setMaxVelocityScalingFactor(1.0);
}

void PlanningClass::goToNamedGoal(std::string &name) {
  move_group.setNamedTarget(name);
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);

  if (!success) // execute
    throw std::runtime_error("No plan found");

  sleep(3.0);
  move_group.move(); // blocking
}