// Imports from standard libraries available
#include "cstring" //Including to convert to c_string
#include "geometry_msgs/Pose.h"
#include "moveit/move_group_interface/move_group_interface.h"
#include "moveit/planning_scene_interface/planning_scene_interface.h"
#include "moveit_msgs/AttachedCollisionObject.h"
#include "moveit_msgs/CollisionObject.h"
#include "moveit_msgs/DisplayRobotState.h" //Aded from cartesian path youtube-dude
#include "moveit_msgs/DisplayTrajectory.h" // Added to visualize trajectory in RVIZ
#include "moveit_msgs/MoveItErrorCodes.h" //Added to include the SUCCESS etc. error codes
#include "moveit_visual_tools/moveit_visual_tools.h" //Adding the moveit Visual tools to show trajectory in RVIZ.
#include "string"
#include "vector"

#include <ros/duration.h>
#include <ros/ros.h>
#include <ros/time.h>
#include <std_msgs/Float64.h>
#include <std_msgs/Float64MultiArray.h>

#include "control_msgs/JointTrajectoryControllerState.h" // Including the
// joint trajectory controller msgs
#include "rosbag/bag.h" //Adding bag file to programatically create bag file of data.
#include "rosbag/view.h"            //For reading the bag files
#include "sensor_msgs/JointState.h" // Including the message types to be
// written to  bag file.

// Putting the class inside a namespace
namespace r7_planning {
class PlanningClass {
public:
  PlanningClass() : move_group(planningGroup) {
    // Setting the initial pose
    target_pose1.orientation.w = 0.707;
    target_pose1.orientation.x = 0.0;
    target_pose1.orientation.y = -0.707;
    target_pose1.orientation.z = 0.0;
    target_pose1.position.x = 0.25;
    target_pose1.position.y = 0.25;
    target_pose1.position.z = 0.3;

    move_group.allowReplanning(true);
    move_group.setNumPlanningAttempts(10);
    move_group.setPoseReferenceFrame("base_link");
    // How many times the solver is allowed to try and plan the pose and the
    // reference frame (?)
  }

  // Adding function to print basic information.
  void basicInfo();

  // The possibilities of setting poses of the robot
  void goToPoseGoal();
  void goToPoseGoal(geometry_msgs::Pose &pose);
  // void goToJointState();
  void cartesianPath();
  void cartesianPathMatlab(std::string &fil);

  void resetValues();
  void goToNamedGoal(std::string &name);
  void runPointsMatlab();

private:
  // Initiations of the things we need
  const std::string planningGroup = "arm";
  // This is the defined MoveIt group for the arm defined preciously

  moveit::planning_interface::MoveGroupInterface move_group;
  // Instance of move_group

  moveit::planning_interface::PlanningSceneInterface virtual_world;
  // Instance of scene

  // const moveit::core::JointModelGroup joint_model_group;
  // Instance of joint group or the use of a raw pointer as in the tutorial
  // beneath
  const robot_state::JointModelGroup *joint_model_group =
      move_group.getCurrentState()->getJointModelGroup(planningGroup);

  moveit::planning_interface::MoveGroupInterface::Plan my_plan;
  // Instance of the created plan

  // Initiating a robot trajectory (?)
  // moveit_msgs::RobotTrajectory trajecplan = my_plan.trajectory_;

  geometry_msgs::Pose target_pose1;
  // Instance of initial spawn pose that is being set in the parameter list of
  // creation above.

  // Moving the trajectory trat is general to be placed here
  moveit_msgs::RobotTrajectory trajectory;

  // Adding bag to store topics inside
  rosbag::Bag bag;

  // #### Creating a publisher for visualizing plans in RViz ####
  //   ros::Publisher display_publisher =
  //       node_handle.advertise<moveit_msgs::DisplayTrajectory>(
  //           "/move_group/display_planned_path", 1, true);
  //   moveit_msgs::DisplayTrajectory display_trajetory;
};

} // namespace r7_planning