#include "planning.hpp"

using namespace r7_planning;

void PlanningClass::basicInfo() {
  // Printing reference frame
  ROS_INFO("NB! Reference frame: %s", move_group.getPlanningFrame().c_str());
  // robot_state::RobotStatePtr roboten = move_group.getCurrentState();
}

void PlanningClass::goToPoseGoal() {
  move_group.setPoseTarget(target_pose1);
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);

  if (!success) // execute
    throw std::runtime_error("No plan found");

  sleep(3.0);
  // Test of visualizing plan continously ###DOES THIS WORK?###
  //   if (1) {
  //     ROS_INFO("Visualizing Plan 1 (again)");
  //     display_trajetory.trajectory_start = my_plan.trajectory_start;
  //     display_trajetory.trajectory.push_back(my_plan.trajectory);
  //     display_publisher.publish(display_trajetory);
  //     sleep(3);
  //   }

  move_group.move(); // blocking
}

void PlanningClass::goToPoseGoal(geometry_msgs::Pose &pose) {
  move_group.setPoseTarget(pose);
  ros::Duration(0.5).sleep();
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);
  // moveit::planning_interface::MoveItErrorCode::SUCCESS  Changed with the
  // other package as above
  /*while (!success) //keep trying until a plan is found
  {

      success = (move_group.plan(my_plan) ==
  moveit::planning_interface::MoveItErrorCode::SUCCESS);
  }*/
  if (!success) // execute
    throw std::runtime_error("No plan found");

  move_group.move(); // blocking
}

// //This must be fixed
// void PlanningClass::goToJointState() {
// From the PR2 tutorial:
// std::vector<double> group_variable_values;
// move_group.getCurrentState()->copyJointGroupPositions(move_group.getCurrentState()->getRobotModel()->getJointModelGroup(move_group.getName()),
// group_variable_values);

//   robot_state::RobotState current_state = *move_group.getCurrentState();
//   // moveit::core::RobotStatePtr current_state =
//   move_group.getCurrentState(); std::vector<double> joint_positions;
//   joint_model_group = current_state.getJointModelGroup(PLANNING_GROUP);
//           joint_positions[0] = -1.0;
//   joint_positions[3] = 0.7;

//   move_group.setJointValueTarget(joint_positions);
//   bool success = (move_group.plan(my_plan)
//   ==moveit::planning_interface::MoveItErrorCode::SUCCESS); if (!success)
//     throw std::runtime_error("No plan found");

//   move_group.move(); // blocking
// }

void PlanningClass::cartesianPath() {
  std::vector<geometry_msgs::Pose> waypoints;
  // waypoints.push_back(target_pose1);
  move_group.setStartStateToCurrentState();

  // Adding a few more points
  geometry_msgs::Pose target_pose2;
  geometry_msgs::Pose target_pose3;
  geometry_msgs::Pose target_pose4;

  target_pose2.orientation.w = 0.001;
  target_pose2.orientation.x = 0.0;
  target_pose2.orientation.y = 1.0;
  target_pose2.orientation.z = 0.0;
  target_pose2.position.x = 0.25;
  target_pose2.position.y = 0.25;
  target_pose2.position.z = 0.1;

  target_pose3.orientation.w = 0.001;
  target_pose3.orientation.x = 0.0;
  target_pose3.orientation.y = 1.0;
  target_pose3.orientation.z = 0.0;
  target_pose3.position.x = 0.3;
  target_pose3.position.y = 0.25;
  target_pose3.position.z = 0.2;

  target_pose4.orientation.w = 0.001;
  target_pose4.orientation.x = 0.0;
  target_pose4.orientation.y = 1.0;
  target_pose4.orientation.z = 0.0;
  target_pose4.position.x = 0.4;
  target_pose4.position.y = 0.25;
  target_pose4.position.z = 0.3;

  // waypoints.push_back(target_pose1);
  waypoints.push_back(target_pose2);
  waypoints.push_back(target_pose3);
  waypoints.push_back(target_pose4);

  move_group.setMaxVelocityScalingFactor(0.75);

  // Printing the waypoints positions one by one
  for (geometry_msgs::Pose tp : waypoints) {
    ROS_INFO("!!! NB !!! Pose: %f, %f, %f", tp.position.x, tp.position.y,
             tp.position.z);
  }

  // We want the Cartesian path to be interpolated at a resolution of 1 cm
  // which is why we will specify 0.01 as the max step in Cartesian
  // translation.  We will specify the jump threshold as 0.0, effectively
  // disabling it. Warning - disabling the jump threshold while operating real
  // hardware can cause large unpredictable motions of redundant joints and
  // could be a safety issue
  moveit_msgs::RobotTrajectory trajectory;
  const double jump_threshold = 0.0;
  const double eef_step = 0.01;
  double fraction = move_group.computeCartesianPath(waypoints, eef_step,
                                                    jump_threshold, trajectory);
  ROS_INFO_STREAM("Percentage of path followed: " << fraction);
  sleep(5);
  // ros::Duration(4).sleep();
  move_group.move();
}

void PlanningClass::resetValues() {
  // set the start state and operational speed
  move_group.setStartStateToCurrentState();
  move_group.setMaxVelocityScalingFactor(1.0);
}

void PlanningClass::makeBox(std::string blk_name, double *pose) {
  moveit_msgs::CollisionObject box;
  // set the relative frame
  box.header.frame_id = move_group.getPlanningFrame();
  box.id = blk_name;

  shape_msgs::SolidPrimitive primitive;
  primitive.type = primitive.BOX;
  primitive.dimensions.resize(3);
  primitive.dimensions[0] = 0.2;
  primitive.dimensions[1] = 0.2;
  primitive.dimensions[2] = 1.0;

  geometry_msgs::Pose box_pose;
  box_pose.orientation.w = 1.0;
  box_pose.position.x = pose[0];
  box_pose.position.y = pose[1];
  box_pose.position.z = pose[2];

  box.primitives.push_back(primitive);
  box.primitive_poses.push_back(box_pose);
  box.operation = box.ADD;

  std::vector<moveit_msgs::CollisionObject> collisionObjects;
  collisionObjects.push_back(box);
  ros::Duration(2).sleep();
  virtual_world.addCollisionObjects(collisionObjects);
  sleep(3.0);
  ROS_INFO_STREAM("Added: " << blk_name);
}

void PlanningClass::addObjects() {
  double box_pose1[3] = {
      0.60,
      -0.67,
      0.0,
  };
  makeBox("block_1", box_pose1);

  double box_pose2[3] = {
      0.0,
      0.77,
      0.0,
  };
  makeBox("block_2", box_pose2);
}

void PlanningClass::removeObjects() {
  std::vector<std::string> object_ids;
  object_ids.push_back("block_1");
  object_ids.push_back("block_2");
  virtual_world.removeCollisionObjects(object_ids);
}