#include "planning.hpp"

int main(int argc, char **argv) {
  // Adding the namespace used to define all functions
  using namespace r7_planning;

  ros::init(argc, argv, "custom_interfacing");
  ros::NodeHandle node_handle;
  ros::AsyncSpinner spinner(2);
  spinner.start();

  if (argc != 2) {
    ROS_INFO(" ");
    ROS_INFO("\tUsage:");
    ROS_INFO(" ");
    ROS_INFO("\trosrun planning run  n");
    return 1;
  }

  // Initializing the planning class
  r7_planning::PlanningClass my_planning_;

  // Initiating a second pose goal
  geometry_msgs::Pose t1;
  geometry_msgs::Pose t2;
  geometry_msgs::Pose t3;
  geometry_msgs::Pose t4;

  //   //Denne funka
  //   t1.orientation.w = 0.001;
  //   t1.orientation.x = 0;
  //   t1.orientation.y = 1.0;
  //   t1.orientation.z = 0;
  //   t1.position.x = 0.2;
  //   t1.position.y = 0.2;
  //   t1.position.z = 0.3;

  t1.orientation.w = 0.001;
  t1.orientation.x = 0;
  t1.orientation.y = 1.0;
  t1.orientation.z = 0;
  t1.position.x = 0.0;
  t1.position.y = 0.4;
  t1.position.z = 0.2;

  // Denne funka også
  //   t2.orientation.w = 0.001;
  //   t2.orientation.x = 0;
  //   t2.orientation.y = 1.0;
  //   t2.orientation.z = 0;
  //   t2.position.x = 0.3;
  //   t2.position.y = 0.25;
  //   t2.position.z = 0.2;

  t2.orientation.w = 0.001;
  t2.orientation.x = 0;
  t2.orientation.y = 1.0;
  t2.orientation.z = 0;
  t2.position.x = 0.3;
  t2.position.y = 0.3;
  t2.position.z = 0.2;

  t3.orientation.w = 0.93;
  t3.orientation.x = 0;
  t3.orientation.y = 0.355;
  t3.orientation.z = 0;
  t3.position.x = 0.4;
  t3.position.y = 0.0;
  t3.position.z = 0.5;

  t4.position.x = 0.4;
  t4.position.y = 0.0;
  t4.position.z = 0.45;
  t4.orientation.w = 0.934722;
  t4.orientation.x = 0;
  t4.orientation.y = 0.355381;
  t4.orientation.z = 0;

  std::string csvfil = "circular_traj.csv";
  std::string csvfil2 = "feed_traj.csv";
  std::string name = "standing";

  int selection = atoi(argv[1]);
  switch (selection) {
    my_planning_.resetValues();
    // Printing basic information
    my_planning_.basicInfo();

  case 1:
    my_planning_.goToPoseGoal(t1);
    break;
  case 2:
    my_planning_.goToPoseGoal(t2);
    break;
  case 3:
    my_planning_.goToPoseGoal(t3);
    break;
  case 4:
    my_planning_.goToPoseGoal(t4);
    break;
  case 5:
    my_planning_.cartesianPath();
    break;
  case 6:
    my_planning_.cartesianPathMatlab(csvfil);
    break;
  case 7:
    my_planning_.cartesianPathMatlab(csvfil2);
    break;
  case 8:
    my_planning_.runPointsMatlab();
    break;
  case 9:
    my_planning_.goToNamedGoal(name);
    break;
  }
  spinner.stop();
  return 0;
}