#include "ros/ros.h"

#include "moveit/move_group_interface/move_group_interface.h" // OR #include "moveit/move_group_interface/move_group_interface.h"
#include "moveit/planning_scene_interface/planning_scene_interface.h"
#include "moveit_msgs/DisplayRobotState.h"
#include "moveit_msgs/DisplayTrajectory.h"

int main(int argc, char **argv) {
  ros::init(argc, argv, "r7_test");
  ros::NodeHandle nodeHandle;
  ros::AsyncSpinner spinner(1);
  spinner.start();

  static const std::string PLANNING_GROUP = "arm";
  moveit::planning_interface::MoveGroupInterface move_group(
      PLANNING_GROUP); // Er dette eigentlig riktig? trur movegroup e DEPRECATED
  moveit::planning_interface::PlanningSceneInterface planning_scene_interface;
  const robot_state::JointModelGroup *joint_model_group =
      move_group.getCurrentState()->getJointModelGroup(PLANNING_GROUP);

  // Setup the planner
  move_group.setPoseReferenceFrame("base_link");
  move_group.setPlannerId("RRTConnectkConfigDefault");
  move_group.setMaxVelocityScalingFactor(0.2);
  move_group.setMaxAccelerationScalingFactor(0.2);
  move_group.setNumPlanningAttempts(10);
  move_group.setStartStateToCurrentState();

  ROS_INFO("NB! Reference frame: %s", move_group.getPlanningFrame().c_str());
  ROS_INFO("NB! Reference frame: %s", move_group.getEndEffectorLink().c_str());

  geometry_msgs::Pose target_pose;
  target_pose.orientation.w = 0.001;
  target_pose.orientation.x = 0.0;
  target_pose.orientation.y = 1.0;
  target_pose.orientation.z = 0.0;
  target_pose.position.x = 0.3;
  target_pose.position.y = 0.3;
  target_pose.position.z = 0.2;
  move_group.setPoseTarget(target_pose);

  moveit::planning_interface::MoveGroupInterface::Plan my_plan;
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);
  ROS_INFO("Plan: %s", success ? "SUCCESS" : "FAILED");
  if (success) {
    ROS_INFO("Moving...Hold on...");
    move_group.move();
  }
  sleep(5);

  


  return 0;
}