#include "planning.hpp"
#include "cstdlib" // Added to be able to use atof, converting character string to float
#include "string"
#include "vector"
#include <iostream>

using namespace bravo_planning;

void PlanningClass::goToPoseGoal(geometry_msgs::Pose &pose) {
  // move_group.setStartStateToCurrentState();
  move_group.setPoseTarget(pose);
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);
  if (!success) // execute
    throw std::runtime_error("No plan found");

  // ros::Duration(1).sleep();
  move_group.execute(my_plan); // blocking
  // move_group.move();
}

void PlanningClass::cartesianPath() {
  std::vector<geometry_msgs::Pose> waypoints;

  // Adding a few more points
  geometry_msgs::Pose target_pose2;
  geometry_msgs::Pose target_pose3;
  geometry_msgs::Pose target_pose4;

  target_pose2.orientation.w = 0.001;
  target_pose2.orientation.x = 0.0;
  target_pose2.orientation.y = 1.0;
  target_pose2.orientation.z = 0.0;
  target_pose2.position.x = 0.3;
  target_pose2.position.y = 0.3;
  target_pose2.position.z = 0.2;

  target_pose3.orientation.w = 0.001;
  target_pose3.orientation.x = 0.0;
  target_pose3.orientation.y = 1.0;
  target_pose3.orientation.z = 0.0;
  target_pose3.position.x = 0.4;
  target_pose3.position.y = 0.0;
  target_pose3.position.z = 0.2;

  target_pose4.orientation.w = 0.001;
  target_pose4.orientation.x = 0.0;
  target_pose4.orientation.y = 1.0;
  target_pose4.orientation.z = 0.0;
  target_pose4.position.x = 0.3;
  target_pose4.position.y = -0.3;
  target_pose4.position.z = 0.2;

  // waypoints.push_back(target_pose1);
  waypoints.push_back(target_pose2);
  waypoints.push_back(target_pose3);
  waypoints.push_back(target_pose4);

  double fraction = move_group.computeCartesianPath(
      waypoints, PlanningClass::eef_step, PlanningClass::jump_threshold,
      PlanningClass::trajectory, PlanningClass::avoid_collisions);
  ROS_INFO_STREAM("Percentage of path followed: " << 100 * fraction);

  // Sending trajectory data to file
  std::ofstream newFile;
  newFile.open("trajectoryPre.csv");
  const char sep = ',';

  if (!newFile) {
    ROS_WARN("Cannot open file..");
  }
  ROS_INFO("File is open, ready to write.");

  // This has information related to each joint, NOT within the cartesian space
  int count = 0;
  for (trajectory_msgs::JointTrajectoryPoint jtp :
       PlanningClass::trajectory.joint_trajectory.points) {
    std::vector<double> accs =
        jtp.accelerations; // Comes from jtp.accelerations as float64[], as
                           // array
    std::vector<double> pos =
        jtp.positions; // Comes from jtp.positions as float64[], an array
    std::vector<double> eff =
        jtp.effort; // Comes from jtp.effort as float64[], an array
    std::vector<double> vel =
        jtp.velocities; // Comes from jtp.velocities as float64[], an array

    double secs = ros::Time::now().toSec();
    ros::Duration rd = jtp.time_from_start;
    secs = rd.toSec();
    ROS_INFO("Point nr. %i, Time: %f , Accs: %f %f %f %f %f %f, Pos: %f %f %f "
             "%f %f %f, Eff: %lu, Vel: %f %f %f %f %f %f",
             count, secs, accs[0], accs[1], accs[2], accs[3], accs[4], accs[5],
             pos[0], pos[1], pos[2], pos[3], pos[4], pos[5], eff.size(), vel[0],
             vel[1], vel[2], vel[3], vel[4], vel[5]);
    count += 1;
    newFile << std::to_string(pos[0]) << sep << std::to_string(pos[1]) << sep
            << std::to_string(pos[2]) << sep << std::to_string(pos[3]) << sep
            << std::to_string(pos[4]) << sep << std::to_string(pos[5])
            << ", Timestamp:" << secs << "\n";
  }

  sleep(5);
  my_plan.trajectory_ = PlanningClass::trajectory;

  if (fraction == 1.0) {
    move_group.execute(my_plan);
    ROS_INFO("All good, 100 percent of path was created");
  } else {
    ROS_WARN("Could not compute the cartesian path :( ");
    ROS_WARN("Make sure that all poses are feasible and that the initial pose "
             "is well-defined!");
  }
  sleep(5);

  newFile.close();
  ros::shutdown();
}

// #### Function for creating trajectory from Matlab points input ####
void PlanningClass::cartesianPathCsv(std::string &fil) {
  std::vector<geometry_msgs::Pose> waypoints;

  // Trying to change planner... Does this work?

  // Reading information from csv file from Matlab
  std::ifstream myFile; // Reading input file
  std::string path = "/home/user/catkin_ws/src/motion_planning/src/" + fil;
  myFile.open(path);
  std::string px, py, pz, ow, ox, oy, oz;

  if (!myFile.is_open()) {
    ROS_WARN("ERROR: File Open");
  }

  while (myFile.good()) {
    geometry_msgs::Pose wp;
    std::getline(myFile, px, ',');
    std::getline(myFile, py, ',');
    std::getline(myFile, pz, ',');
    std::getline(myFile, ow, ',');
    std::getline(myFile, ox, ',');
    std::getline(myFile, oy, ',');
    std::getline(myFile, oz, '\n');

    wp.position.x = atof(px.c_str());
    wp.position.y = atof(py.c_str());
    wp.position.z = atof(pz.c_str());
    wp.orientation.w = atof(ow.c_str());
    wp.orientation.x = atof(ox.c_str());
    wp.orientation.y = atof(oy.c_str());
    wp.orientation.z = atof(oz.c_str());
    waypoints.push_back(wp);
  }

  // Writing the input waypoints to another file to check if something might be
  // wrong with formatting.
  //   std::ofstream utwp;
  //   utwp.open("utwp.csv");

  //   // Printing the waypoints positions one by one
  //   for (geometry_msgs::Pose tp : waypoints) {
  //     ROS_INFO("!!! NB !!! Pose: %f, %f, %f", tp.position.x, tp.position.y,
  //              tp.position.z);
  //     ROS_INFO("!!! NB !!! Quarterion: %f, %f, %f, %f", tp.orientation.x,
  //              tp.orientation.y, tp.orientation.z, tp.orientation.w);

  //     utwp << std::to_string(tp.position.x) << "," << tp.position.y << ","
  //          << tp.position.z << "," << tp.orientation.w << "," <<
  //          tp.orientation.x
  //          << "," << tp.orientation.y << "," << tp.orientation.z << "\n";
  //   }

  double fraction = move_group.computeCartesianPath(
      waypoints, PlanningClass::eef_step, PlanningClass::jump_threshold,
      PlanningClass::trajectory, PlanningClass::avoid_collisions);
  ROS_INFO_STREAM("Percentage of path followed: " << 100 * fraction);

  // #### Trajectory information ####
  // vector of vectors for each of joint angle corresponding to point of
  // trajectory
  std::vector<std::vector<double>> anglepoints;

  // Sending trajectory data to file
  std::ofstream newFile;
  newFile.open("trajectory.csv");
  const char sep = ',';

  if (!newFile) {
    ROS_WARN("Cannot open file..");
  }
  ROS_INFO("File is open, ready to write.");

  // This has information related to each joint, NOT within the cartesian space
  int count = 0;
  ros::Duration dt(0.0);
  ros::Duration increase(0.01);
  for (trajectory_msgs::JointTrajectoryPoint jtp :
       PlanningClass::trajectory.joint_trajectory.points) {
    std::vector<double> accs =
        jtp.accelerations; // Comes from jtp.accelerations as float64[], as
                           // array
    std::vector<double> pos =
        jtp.positions; // Comes from jtp.positions as float64[], an array
    std::vector<double> eff =
        jtp.effort; // Comes from jtp.effort as float64[], an array
    std::vector<double> vel =
        jtp.velocities; // Comes from jtp.velocities as float64[], an array

    double secs = ros::Time::now().toSec();
    ros::Duration rd = jtp.time_from_start;
    secs = rd.toSec();
    // ROS_INFO("Point nr. %i, Time: %f , Accs: %f %f %f %f %f %f, Pos: %f %f %f
    // "
    //          "%f %f %f, Eff: %lu, Vel: %f %f %f %f %f %f",
    //          count, secs, accs[0], accs[1], accs[2], accs[3], accs[4],
    //          accs[5], pos[0], pos[1], pos[2], pos[3], pos[4], pos[5],
    //          eff.size(), vel[0], vel[1], vel[2], vel[3], vel[4], vel[5]);
    count += 1;
    anglepoints.push_back(pos);
    newFile << std::to_string(pos[0]) << sep << std::to_string(pos[1]) << sep
            << std::to_string(pos[2]) << sep << std::to_string(pos[3]) << sep
            << std::to_string(pos[4]) << sep << std::to_string(pos[5])
            << ", Timestamp:" << secs << "\n";

    // ## Testing to add timestamp to each waypoint manually since they are
    // defined as not strictly increasing. Increaseing by 0.01
    jtp.time_from_start = dt;
    dt += increase;
    // ROS_INFO("Timestamp new: %f", dt.toSec());
  }

  // #### Execution ####
  sleep(3);
  my_plan.trajectory_ = PlanningClass::trajectory;
  ROS_INFO("Trajectory uploaded.");

  if (fraction == 1.0) {
    move_group.execute(my_plan);
    ROS_INFO("All good, 100 percent of path was created");
  } else {
    ROS_WARN("Could not compute the cartesian path :( ");
    ROS_WARN("Make sure that all poses are feasible and that the initialpose "
             "is well-defined!");

    // Trying to add the movement to the first waypoint
    ROS_INFO("Moving to initial state of waypoints which is as follows:");
    ROS_INFO("Pos: %f ,%f, %f and Ori: %f, %f, %f, %f", waypoints[0].position.x,
             waypoints[0].position.y, waypoints[0].position.z,
             waypoints[0].orientation.w, waypoints[0].orientation.x,
             waypoints[0].orientation.y, waypoints[0].orientation.z);

    goToPoseGoal(waypoints[0]);
    sleep(3);
    waypoints.erase(waypoints.begin());

    // #### Replanning for the remaining waypoints ####

    double frac = move_group.computeCartesianPath(
        waypoints, PlanningClass::eef_step, PlanningClass::jump_threshold,
        PlanningClass::trajectory, avoid_collisions);
    ROS_INFO_STREAM("Percentage of new path followed: " << 100 * frac);

    sleep(3);
    my_plan.trajectory_ = PlanningClass::trajectory;

    if (frac == 1.0) {
      move_group.execute(my_plan);
      ROS_INFO("All good, 100 percent of path was created");
    } else {
      ROS_WARN("Could not compute the cartesian path :( ");
      ROS_WARN("Make sure that all waypoints are feasible! Shutting down...");
    }
    newFile.close();
    myFile.close();
    sleep(5);
  }

  // // #### If nothing works, try to walk through to each angle of the joints
  // -> This is just for testing purposes.. (of the trajectory generated) ####

  //   int cunt = 0;
  //   for (std::vector<double> joint_positions : anglepoints) {
  //     move_group.setJointValueTarget(joint_positions);
  //     ROS_INFO("Joint angles for traj part %i of
  //     %lu",cunt,anglepoints.size()); bool success =
  //         (move_group.plan(my_plan) ==
  //         moveit_msgs::MoveItErrorCodes::SUCCESS);
  //     if (!success)
  //       throw std::runtime_error("No plan found");
  //     sleep(2);
  //     move_group.move(); // blocking
  //   }

  // Instead of the above it is possible to iterate through each 10th or
  // something joint angles.
  //   for (std::size_t it = 0; it < anglepoints.size(); it += 10) {
  //     move_group.setJointValueTarget(anglepoints[it]);
  //     ROS_INFO("Joint angles for traj part %lu of %lu", it,
  //     anglepoints.size()); bool success =
  //         (move_group.plan(my_plan) ==
  //         moveit_msgs::MoveItErrorCodes::SUCCESS);
  //     if (!success)
  //       throw std::runtime_error("No plan found");
  //     sleep(2);
  //     move_group.move(); // blocking
  //   }

  ros::shutdown();
}

// Testing of poses from the Matlab inputs
void PlanningClass::runPosesCsv(std::string &fil, int interval) {

  std::vector<geometry_msgs::Pose> waypoints;
  // Reading information from csv file from Matlab
  std::ifstream myFile; // Reading input file
  myFile.open("/home/user/catkin_ws/src/motion_planning/src/" + fil);
  std::string px, py, pz, ow, ox, oy, oz;

  if (!myFile.is_open()) {
    ROS_WARN("ERROR: File Open");
  }

  while (myFile.good()) {
    geometry_msgs::Pose wp;
    std::getline(myFile, px, ',');
    std::getline(myFile, py, ',');
    std::getline(myFile, pz, ',');
    std::getline(myFile, ow, ',');
    std::getline(myFile, ox, ',');
    std::getline(myFile, oy, ',');
    std::getline(myFile, oz, '\n');
    // ROS_INFO("####Pose: %s, %s, %s; %s, %s, %s, %s", px.c_str(),
    // py.c_str(), pz.c_str(), ow.c_str(), ox.c_str(), oy.c_str(),
    // oz.c_str());

    wp.position.x = atof(px.c_str());
    wp.position.y = atof(py.c_str());
    wp.position.z = atof(pz.c_str());
    wp.orientation.w = atof(ow.c_str());
    wp.orientation.x = atof(ox.c_str());
    wp.orientation.y = atof(oy.c_str());
    wp.orientation.z = atof(oz.c_str());
    waypoints.push_back(wp);
  }

  // Running the points specified by interval.
  for (std::size_t it = 0; it < waypoints.size(); it += interval) {
    goToPoseGoal(waypoints[it]);
    // sleep(1);
    getPose();
    ROS_INFO("Pose nr: %li", it);
  }

  // Or running points one by one.
  //   int nr = 0;
  //   for (geometry_msgs::Pose poss : waypoints) {
  //     goToPoseGoal(poss);
  //     ROS_INFO("Sleeping before next pose. Please wait...");
  //     ROS_WARN("This is Pose nr: %i", nr);
  //     sleep(1);
  //     nr++;
  //   }
}

void PlanningClass::runPoseCsv(std::string &fil, int which) {
  std::vector<geometry_msgs::Pose> waypoints;
  // Reading information from csv file from Matlab
  std::ifstream myFile; // Reading input file
  myFile.open("/home/user/catkin_ws/src/motion_planning/src/" + fil);
  std::string px, py, pz, ow, ox, oy, oz;

  if (!myFile.is_open()) {
    ROS_WARN("ERROR: File Open");
  }

  while (myFile.good()) {
    geometry_msgs::Pose wp;
    std::getline(myFile, px, ',');
    std::getline(myFile, py, ',');
    std::getline(myFile, pz, ',');
    std::getline(myFile, ow, ',');
    std::getline(myFile, ox, ',');
    std::getline(myFile, oy, ',');
    std::getline(myFile, oz, '\n');
    // ROS_INFO("####Pose: %s, %s, %s; %s, %s, %s, %s", px.c_str(),
    // py.c_str(), pz.c_str(), ow.c_str(), ox.c_str(), oy.c_str(),
    // oz.c_str());

    wp.position.x = atof(px.c_str());
    wp.position.y = atof(py.c_str());
    wp.position.z = atof(pz.c_str());
    wp.orientation.w = atof(ow.c_str());
    wp.orientation.x = atof(ox.c_str());
    wp.orientation.y = atof(oy.c_str());
    wp.orientation.z = atof(oz.c_str());
    waypoints.push_back(wp);
  }
  goToPoseGoal(waypoints[which]);
}

void PlanningClass::goToNamedGoal(std::string &name) {
  move_group.setNamedTarget(name);
  bool success =
      (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);

  if (!success) // execute
    throw std::runtime_error("No plan found");

  sleep(3.0);
  move_group.execute(my_plan); // blocking
}

void PlanningClass::jointsCSV(std::string &fil) {
  // Reading information from csv file from Matlab
  std::ifstream myFile; // Reading input file
  myFile.open("/home/user/catkin_ws/src/motion_planning/src/" + fil);
  std::string j1, j2, j3, j4, j5, j6;
  std::vector<double> jTargets;

  if (!myFile.is_open()) {
    ROS_WARN("ERROR: File Open");
  }

  while (myFile.good()) {
    std::getline(myFile, j1, ',');
    std::getline(myFile, j2, ',');
    std::getline(myFile, j3, ',');
    std::getline(myFile, j4, ',');
    std::getline(myFile, j5, ',');
    std::getline(myFile, j6, '\n');

    jTargets.push_back(atof(j1.c_str()));
    jTargets.push_back(atof(j2.c_str()));
    jTargets.push_back(atof(j3.c_str()));
    jTargets.push_back(atof(j4.c_str()));
    jTargets.push_back(atof(j5.c_str()));
    jTargets.push_back(atof(j6.c_str()));
    move_group.setJointValueTarget(jTargets);
  }

  //   bool success =
  //       (move_group.plan(my_plan) == moveit_msgs::MoveItErrorCodes::SUCCESS);
  //   if (!success) {
  //     throw std::runtime_error("No plan found for the joint angle");
  //   } else {
  //     sleep(2);
  //     move_group.execute(my_plan);
  //   }
  move_group.move();
}

void PlanningClass::getPose() {
  geometry_msgs::PoseStamped current = move_group.getCurrentPose();

  ROS_INFO("####Pose: %f, %f, %f; %f, %f, %f, %f ####", current.pose.position.x,
           current.pose.position.y, current.pose.position.z,
           current.pose.orientation.w, current.pose.orientation.x,
           current.pose.orientation.y, current.pose.orientation.z);
}
