// Imports from standard libraries available
#include "cstring" //Including to convert to c_string
#include "geometry_msgs/Pose.h"
#include "moveit/move_group_interface/move_group_interface.h"
#include "moveit/planning_scene_interface/planning_scene_interface.h"
#include "moveit_msgs/AttachedCollisionObject.h"
#include "moveit_msgs/CollisionObject.h"
#include "moveit_msgs/DisplayRobotState.h" //Aded from cartesian path youtube-dude
#include "moveit_msgs/DisplayTrajectory.h" // Added to visualize trajectory in RVIZ
#include "moveit_msgs/MoveItErrorCodes.h" //Added to include the SUCCESS etc. error codes
#include "moveit_visual_tools/moveit_visual_tools.h" //Adding the moveit Visual tools to show trajectory in RVIZ.
#include "string"
#include "vector"

#include <ros/duration.h>
#include <ros/ros.h>
#include <ros/time.h>
#include <std_msgs/Float64.h>
#include <std_msgs/Float64MultiArray.h>

#include "control_msgs/JointTrajectoryControllerState.h" // Including the
// joint trajectory controller msgs
#include "rosbag/bag.h" //Adding bag file to programatically create bag file of data.
#include "rosbag/view.h"            //For reading the bag files
#include "sensor_msgs/JointState.h" // Including the message types to be
// written to  bag file.

// Putting the class inside a namespace
namespace bravo_planning {
class PlanningClass {
public:
  PlanningClass() : move_group(planningGroup) {

    move_group.allowReplanning(true);
    move_group.setNumPlanningAttempts(5);
    move_group.setPlanningTime(5);
    move_group.setPoseReferenceFrame("base_link");
    move_group.setGoalOrientationTolerance(0.01);
    move_group.setGoalPositionTolerance(0.01);
    move_group.setMaxVelocityScalingFactor(0.8);
    move_group.setPlannerId("RRTkConfigDefault");
  }

  void goToPoseGoal(geometry_msgs::Pose &pose);
  void cartesianPath();
  void cartesianPathCsv(std::string &fil);
  void goToNamedGoal(std::string &name);
  void runPosesCsv(std::string &fil, int interval);
  void runPoseCsv(std::string &fil, int which);
  void jointsCSV(std::string &fil);
  void getPose();

private:
  const std::string planningGroup = "arm";
  // This is the defined MoveIt group for the arm defined preciously

  moveit::planning_interface::MoveGroupInterface move_group;
  // Instance of move_group

  moveit::planning_interface::PlanningSceneInterface virtual_world;
  // Instance of scene

  // const moveit::core::JointModelGroup joint_model_group;
  // Instance of joint group or the use of a raw pointer as in the tutorial
  // beneath
  const robot_state::JointModelGroup *joint_model_group =
      move_group.getCurrentState()->getJointModelGroup(planningGroup);

  moveit::planning_interface::MoveGroupInterface::Plan my_plan;
  // Instance of the created plan

  // Moving the trajectory trat is general to be placed here
  moveit_msgs::RobotTrajectory trajectory;

  // Adding bag to store topics inside
  rosbag::Bag bag;

  // Other parameters used in Cartesian path planner etc.
  const double jump_threshold = 0.0;
  const double eef_step = 0.01; // 10 cm
  bool avoid_collisions =
      false; // trying to avoid obstacles if any inserted in virtual environment
};

} // namespace bravo_planning