#include "planning.hpp"

int main(int argc, char **argv) {
  // Adding the namespace used to define all functions
  using namespace bravo_planning;

  ros::init(argc, argv, "custom_interfacing");
  ros::NodeHandle node_handle;
  ros::AsyncSpinner spinner(2);
  spinner.start();

  if (argc != 2 & argc != 3) {
    ROS_INFO(" ");
    ROS_INFO("\tUsage:");
    ROS_INFO(" ");
    ROS_INFO("\trosrun planning run  n m");
    ROS_INFO("\t Where n is the selection in swith and m is point to be "
             "examined of implemented csv points");
    return 1;
  }

  // Initializing the planning class
  bravo_planning::PlanningClass my_planning_;

  // Initiating a second pose goal
  geometry_msgs::Pose t1;
  geometry_msgs::Pose t2;
  geometry_msgs::Pose t3;

  t1.orientation.w = 0.001;
  t1.orientation.x = 0;
  t1.orientation.y = 1.0;
  t1.orientation.z = 0;
  t1.position.x = 0.0;
  t1.position.y = 0.4;
  t1.position.z = 0.2;

  t2.orientation.w = 0.001;
  t2.orientation.x = 0;
  t2.orientation.y = 1.0;
  t2.orientation.z = 0;
  t2.position.x = 0.3;
  t2.position.y = 0.3;
  t2.position.z = 0.2;

  t3.orientation.w = 0.93;
  t3.orientation.x = 0;
  t3.orientation.y = 0.355;
  t3.orientation.z = 0;
  t3.position.x = 0.4;
  t3.position.y = 0.0;
  t3.position.z = 0.5;

  std::string clean = "circular_traj.csv";
  std::string feed = "feed_traj.csv";
  std::string grab = "grab_traj.csv";
  std::string joints = "joints.csv";
  std::string stand = "standing";
  std::string lay = "laying";
  int choice = atoi(argv[2]);

  int selection = atoi(argv[1]);
  switch (selection) {

  case 1:
    my_planning_.goToPoseGoal(t1);
    break;
  case 2:
    my_planning_.goToPoseGoal(t2);
    break;
  case 3:
    my_planning_.goToNamedGoal(stand);
    break;
  case 4:
    my_planning_.goToNamedGoal(lay);
    break;
  case 6:
    my_planning_.cartesianPathCsv(clean);
    break;
  case 7:
    my_planning_.runPosesCsv(clean, choice);
    break;
  case 8:
    my_planning_.runPoseCsv(clean, choice);
    break;
  case 9:
    my_planning_.cartesianPathCsv(feed);
    break;
  case 10:
    my_planning_.runPosesCsv(feed, choice);
    break;
  case 11:
    my_planning_.runPoseCsv(feed, choice);
    break;
  case 12:
    my_planning_.cartesianPathCsv(grab);
    break;
  case 13:
    my_planning_.runPosesCsv(grab, choice);
    break;
  case 14:
    my_planning_.runPoseCsv(grab, choice);
    break;
  case 15:
    my_planning_.jointsCSV(joints);
    break;
  case 16:
    my_planning_.getPose();
    break;
  }
  spinner.stop();
  return 0;
}
